#ifndef _PROBITML_CLASS_
#define _PROBITML_CLASS_

#define WANT_STREAM
#define WANT_MATH

#include <iostream>
#include <valarray>
#include "newmatio.h"    // need matrix output routines
#include "newmatnl.h"    // need matrix newton routines
#include "ProbitRegression.h"	// this is here for access to the NormalProb routines

using namespace std;  //introduces namespace std

class ProbitML_LL : public LL_D_FI
{
   ColumnVector D;				// derivatives of loglikelihood
   SymmetricMatrix D2;			// second derivatives
   ColumnVector p;				// the parameters
   valarray<double> x, N, r;				// access to the data
   int nobs;					// and nobs and npar that will be passed in
   int npar;
	
public:

   ProbitML_LL(const valarray<double> x_in, const valarray<double> N_in, const valarray<double> r_in,
   	const ColumnVector& p_in, const int nobs_in)
   	: x(x_in), N(N_in), r(r_in), p(p_in), nobs(nobs_in) {}  
   	// arguments are x, N, r, the parameter vector, and the number of observations

   void Set(const ColumnVector& p_set) { // set parameter values
      p = p_set;
   }

   bool IsValid();                 // are parameters valid
   Real LogLikelihood();           // return the loglikelihood
   ReturnMatrix Derivatives();     // derivatives of log-likelihood
   ReturnMatrix FI();              // Fisher Information matrix
};

bool ProbitML_LL::IsValid() {
	 return true;
}

Real ProbitML_LL::LogLikelihood() {
	Real LL;          // the log likelihood
	npar = 2;
	D.ReSize(npar);		// set the size of D
	D2.ReSize(npar);	// and D2
	
	double alpha0, beta;	// local variables: alpha, beta, and
	valarray<double> Y(nobs), P(nobs), Q(nobs), h(nobs), prop(nobs);	// derivative intermediates
	double dldalpha, dldbeta, d2ldalpha2, d2ldalphabeta, d2ldbeta2;
	int j;

	alpha0 = p(1);	// extract alpha and beta from the calling parameter list
	beta = p(2);	// and give them local names
	Y = alpha0 + (beta*x);	// it's convenient to compute P in pieces
	P = NormalProbList(Y, nobs);
	Q = 1.0-P;				// and Q, for derivatives later
	LL = 0;
	for (j = 0; j < nobs;  j++) {
		LL += r[j]*log(P[j]) + (N[j]-r[j])*log(Q[j]);	// the loglikelihood
	}													// [regret no log(DblList)]
	
// gradient and hessian from B&J pp. 53-56
	h = (1/sqrt(2*3.1415927))*exp((-0.5)*Y*Y);
	prop = r/N;
	dldalpha = (+1)*((N*(prop - P)*h)/(P*Q)).sum();
	dldbeta = (+1)*((N*(prop - P)*h*x)/(P*Q)).sum();

	D(1) = dldalpha;	// the matrix routines want the derivatives as a column vector
	D(2) = dldbeta;	

	if (wg) {    // do only if second derivatives wanted
		d2ldalpha2 = (-1)*((((N*h*(prop-P))/(P*Q))*
							((h/Q)-(h/P)-Y)).sum() - ((N*h*h)/(P*Q)).sum());
		d2ldalphabeta = (-1)*((((N*h*(prop-P))/(P*Q))*
							((h/Q)-(h/P)-Y)*x).sum() - ((N*h*h*x)/(P*Q)).sum());
		d2ldbeta2 = (-1)*((((N*h*(prop-P))/(P*Q))*
							((h/Q)-(h/P)-Y)*x*x).sum() - ((N*h*h*x*x)/(P*Q)).sum());
					// below, for a SymmetricMatrix only load three elements
		D2 << d2ldalpha2 << d2ldalphabeta << d2ldbeta2;
		}
// the four lines below are for debugging, but left on here as they don't hurt
   	cout << "parameters " << p << endl;
   	cout << "LL " << LL << endl;
	cout << "D " << D;
	if (wg) cout << "D2 " << D2;

   return LL;
}

// "Return" routines are required by the NEWMATNL structure, 
// but here they simply pass back already-computed components of the object
ReturnMatrix ProbitML_LL::Derivatives() { 
	return D; 
}

ReturnMatrix ProbitML_LL::FI() {
   if (!wg) cout << endl << "unexpected call of FI" << endl;
   return D2;
}

#endif /* _PROBITML_CLASS_ */
